@@ -33,7 +33,7 @@ template<class T>
 struct __argsort_comparer {
     const T* x;
     __argsort_comparer(const T* x) { this->x = x; }
-    bool operator()(ssize_t i, ssize_t j) const {
+    bool operator()(Py_ssize_t i, Py_ssize_t j) const {
         return this->x[i] <  this->x[j] ||
               (this->x[i] == this->x[j] && i < j);
     }
@@ -56,11 +56,11 @@ struct __argsort_comparer {
  *  @param stable use a stable sorting algorithm? (slower)
  */
 template<class T>
-void Cargsort(ssize_t* ret, const T* x, ssize_t n, bool stable=true)
+void Cargsort(Py_ssize_t* ret, const T* x, Py_ssize_t n, bool stable=true)
 {
     if (n <= 0) throw std::domain_error("n <= 0");
 
-    for (ssize_t i=0; i<n; ++i)
+    for (Py_ssize_t i=0; i<n; ++i)
         ret[i] = i;
 
     if (stable)
@@ -92,19 +92,19 @@ void Cargsort(ssize_t* ret, const T* x, ssize_t n, bool stable=true)
  *  @param buf optional working buffer of size >= k+1, will be overwritten
  */
 template<class T>
-ssize_t Cargkmin(const T* x, ssize_t n, ssize_t k, ssize_t* buf=NULL)
+Py_ssize_t Cargkmin(const T* x, Py_ssize_t n, Py_ssize_t k, Py_ssize_t* buf=NULL)
 {
-    ssize_t* idx;
+    Py_ssize_t* idx;
 
     if (n <= 0)   throw std::domain_error("n <= 0");
     if (k >= n)   throw std::domain_error("k >= n");
 
     k += 1;
-    if (!buf) idx = new ssize_t[k];
+    if (!buf) idx = new Py_ssize_t[k];
     else      idx = buf;
 
-    for (ssize_t i=0; i<k; ++i) {
-        ssize_t j = i;
+    for (Py_ssize_t i=0; i<k; ++i) {
+        Py_ssize_t j = i;
         idx[i] = i;
         while (j > 0 && x[i] < x[idx[j-1]]) {
             idx[j] = idx[j-1];
@@ -113,10 +113,10 @@ ssize_t Cargkmin(const T* x, ssize_t n, ssize_t k, ssize_t* buf=NULL)
         idx[j] = i;
     }
 
-    for (ssize_t i=k; i<n; ++i) {
+    for (Py_ssize_t i=k; i<n; ++i) {
         if (x[idx[k-1]] <= x[i])
             continue;
-        ssize_t j = k-1;
+        Py_ssize_t j = k-1;
         idx[k-1] = i;
         while (j > 0 && x[i] < x[idx[j-1]]) {
             idx[j] = idx[j-1];
@@ -126,7 +126,7 @@ ssize_t Cargkmin(const T* x, ssize_t n, ssize_t k, ssize_t* buf=NULL)
     }
 
 
-    ssize_t ret = idx[k-1];
+    Py_ssize_t ret = idx[k-1];
 
     if (!buf) delete [] idx;
 
